;*******************************************************
; BESSEL J0 FUNCTION
;
; Input: 	->st(0)=x
; Output:	->st(0)=f(x), st(1)=x
;
;*******************************************************
Bessj0 proc
	
	fld st(0)			; ->st(0)=x, st(1)=x
	fabs		 		; ->st(0)=abs(x), st(1)=x
	
	fcom polynomial[0] 	; We compare abs(x) with 8.0
	fwait
	fstsw ax			
	fwait
	sahf				
	jnb @@greater		; If abs(x) <! 8.0 we jump to the second part of the algorithm
	
	fstp st(0)			; We pop st(0)
	fld st				; -> st(0)=x, st(1)=x
	fmul st,st(1)		; -> st(0)=y=x*x; st(1)=x
	
	;----------------------------------------------------------------------------------
	; ANS1 CALCULATION
	;----------------------------------------------------------------------------------
	fld polynomial[8]	; -> st(0)=-184.9052456; st(1)=y; st(2)=x
	fmul st,st(1)		; -> st(0)=y*(-184.9052456); st(1)=y; st(2)=x
	fld polynomial[16]	; -> st(0)=77392.33017; st(1)=y*(-184.9052456); st(2)=y; st(3)=x
	faddp st(1),st		; -> st(0)=77392.33017+(y*(-184.9052456)); st(1)=y; st(2)=x
	fmul st,st(1)		; -> st(0)=y*(77392+(y*(-184.9052456))); st(1)=y; st(2)=x
	fld polynomial[24]	; -> st(0)=-11214424.18; st(1)=y*(77392+(y*(-184.9052456))); st(2)=y; st(3)=x
	faddp st(1),st		; -> st(0)=-11214424.18+(y*(77392+(y*(-184.9052456)))); st(1)=y; st(2)=x
	fmul st,st(1)		; -> st(0)=y*(-11214424.18+(y*(77392+(y*(-184.9052456))))); st(1)=y; st(2)=x
	fld polynomial[32]	; -> st(0)=651619640.7; st(1)=y*(-11214424.18+(y*(77392+(y*(-184.9052456))))); st(2)=y; st(3)=x
	faddp st(1),st		; -> st(0)=651619640.7+(y*(-11214424.18+(y*(77392+(y*(-184.9052456)))))); st(1)=y; st(2)=x
	fmul st,st(1)		; -> st(0)=y*(651619640.7+(y*(-11214424.18+(y*(77392+(y*(-184.9052456))))))); st(1)=y; st(2)=x
	fld polynomial[40]	; -> st(0)=-13362590354.0; st(1)=y*(651619640.7+(y*(-11214424.18+(y*(77392+(y*(-184.9052456))))))); st(2)=y; st(3)=x
	faddp st(1),st		; -> st(0)=-13362590354.0+(y*(651619640.7+(y*(-11214424.18+(y*(77392+(y*(-184.9052456)))))))); st(1)=y; st(2)=x
	fmul st,st(1)		; -> st(0)=y*(-13362590354.0+(y*(651619640.7+(y*(-11214424.18+(y*(77392+(y*(-184.9052456))))))))); st(1)=y; st(2)=x
	fld polynomial[48]	; -> st(0)=57568490574.0; st(1)=y*(-13362590354.0+(y*(651619640.7+(y*(-11214424.18+(y*(77392+(y*(-184.9052456))))))))); st(2)=y; st(3)=x
	faddp st(1),st		; -> st(0)=57568490574.0+y*(-13362590354.0+(y*(651619640.7+(y*(-11214424.18+(y*(77392+(y*(-184.9052456))))))))); st(1)=y; st(2)=x
	;----------------------------------------------------------------------------------
	; END ANS1 CALCULATION
	;----------------------------------------------------------------------------------
	
	;----------------------------------------------------------------------------------
	; ANS2 CALCULATION
	;----------------------------------------------------------------------------------
	fxch st(1)			; -> st(0)=y; st(1)=ans1; st(2)=x
	fld1				; -> st(0)=1.0; st(1)=y; st(2)=ans1; st(3)=x
	fmul st,st(1)		; -> st(0)=y*1.0; st(1)=y; st(2)=ans1; st(3)=x
	fld polynomial[56]	; -> st(0)=267.8532712; st(1)=y*1.0; st(2)=y; st(3)=ans1; st(4)=x
	faddp st(1),st		; -> st(0)=267.8532712+(y*1.0); st(1)=y; st(2)=ans1; st(3)=x
	fmul st,st(1)		; -> st(0)=y*(267.8532712+(y*1.0)); st(1)=y; st(2)=ans1; st(3)=x
	fld polynomial[64]	; -> st(0)=59272.64853; st(1)=y*(267.8532712+(y*1.0)); st(2)=y; st(3)=ans1; st(4)=x
	faddp st(1), st		; -> st(0)=59272.64853+(y*(267.8532712+(y*1.0))); st(1)=y; st(2)=ans1; st(3)=x
	fld polynomial[72]	; -> st(0)=9494680.718; st(1)=y*(59272.64853+(y*(267.8532712+(y*1.0)))); st(2)=y; st(3)=ans1; st(4)=x
	fmul st,st(1)		; -> st(0)=y*(59272.64853+(y*(267.8532712+(y*1.0)))); st(1)=y; st(2)=ans1; st(3)=x
	faddp st(1), st		; -> st(0)=9494680.718+(y*(59272.64853+(y*(267.8532712+(y*1.0))))); st(1)=y; st(2)=ans1; st(2)=x
	fmul st,st(1)		; -> st(0)=y*(9494680.718+(y*(59272.64853+(y*(267.8532712+(y*1.0)))))); st(1)=y; st(2)=ans1; st(3)=x
	fld polynomial[80]	; -> st(0)=1029532985.0; st(1)=y*(9494680.718+(y*(59272.64853+(y*(267.8532712+(y*1.0)))))); st(2)=y; st(3)=ans1; st(4)=x
	faddp st(1), st		; -> st(0)=1029532985.0+(y*(9494680.718+(y*(59272.64853+(y*(267.8532712+(y*1.0))))))); st(1)=y; st(2)=ans1; st(3)=x
	fmul st, st(1)		; -> st(0)=y*(1029532985.0+(y*(9494680.718+(y*(59272.64853+(y*(267.8532712+(y*1.0)))))))); st(1)=y; st(2)=ans1; st(3)=x
	fld polynomial[88]	; -> st(0)=57568490441.0; st(1)=y*(1029532985.0+(y*(9494680.718+(y*(59272.64853+(y*(267.8532712+(y*1.0)))))))); st(2)=y; st(3)=ans1; st(4)=x
	faddp st(1),st		; -> st(0)=57568490441.0 + (y*(1029532985.0+(y*(9494680.718+(y*(59272.64853+(y*(267.8532712+(y*1.0))))))))); st(1)=y; st(2)=ans1; st(3)=x
	;----------------------------------------------------------------------------------
	; END ANS2 CALCULATION
	;----------------------------------------------------------------------------------
	
	;FPU state before calculate ans1 and ans2
	; -> st(0)=ans2; st(1)=y; st(2)=ans1; st(3)=x
	fxch st(1)			; -> st(0)=y; st(1)=ans2; st(2)=ans1; st(3)=x
	fstp st(0)			; Pop st(0)
	fdivp st(1),st		; -> st(0)=ans1/ans2; st(1)=x
	jmp @@end
	
	;If abs(x) >= 8.0
	@@greater::
	; FPU state
	; -> st(0)=ax; st(1)=x
	fld polynomial[0]		; -> st(0)=8.0; st(1)=ax; st(2)=x
	fwait
	fdiv st,st(1)			; -> st(0)=8.0/ax; st(1)=ax; st(2)=x
	fld st					; -> st(0)=z; st(1)=z; st(2)=ax; st(3)=x
	fwait
	fmul st,st(1)			; -> st(0)=z*z; st(1)=z; st(2)=ax; st(3)=x
	fld st(2)				; -> st(0)=ax; st(1)=y; st(2)=z; st(3)=ax; st(4)=x
	fwait
	fld polynomial[96]		; -> st(0)=0.785398164; st(1)=ax; st(2)=y; st(3)=z; st(4)=ax; st(5)=x
	fwait
	fsubp st(1),st			; -> st(0)=ax-0.785398164; st(1)=y; st(2)=z; st(3)=ax; st(4)=x
	fxch st(1)				; -> st(0)=y; st(1)=xx; st(2)=z; st(3)=ax; st(4)=x
	
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;THIS IS THE CONFLICT LINE. When I remove this line function result is not ok. It return 0
	invoke MessageBox,0, sstr$(1500000), 0 ,MB_OK
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
	;----------------------------------------------------------------------------------
	; ANS1 CALCULATION
	;----------------------------------------------------------------------------------
	fld polynomial[104]	 	; -> st(0)=0.2093887211e-6; st(1)=y; st(2)=xx; st(3)=z; st(4)=ax; st(5)=x
	fwait
	fmul st,st(1)			; -> st(0)=y*0.2093887211e-6; st(1)=y; st(2)=xx; ;st(3)=z; st(4)=ax; st(5)=x
	fld polynomial[112]		; -> st(0)=-0.2073370639e-5; st(1)=y*0.2093887211e-6; st(2)=y; st(3)=xx; st(4)=z; st(5)=ax; st(6)=x
	fwait
	faddp st(1),st			; -> st(0)=-0.2073370639e-5+y*0.2093887211e-6; st(1)=y; st(2)=xx; st(3)=z; st(4)=ax; st(5)=x
	fmul st,st(1)			; -> st(0)=y*(-0.2073370639e-5+y*0.2093887211e-6); st(1)=y; st(2)=xx; st(3)=z; st(4)=ax; st(5)=x
	fld polynomial[120]		; -> st(0)=0.2734510407e-4; st(1)=y*(-0.2073370639e-5+y*0.2093887211e-6); st(2)=y; st(3)=xx; st(4)=z; st(5)=ax; st(6)=x
	fwait
	faddp st(1),st			; -> st(0)=0.2734510407e-4+y*(-0.2073370639e-5+y*0.2093887211e-6); st(1)=y; st(2)=xx; st(3)=z; st(4)=ax; st(5)=x
	fmul st,st(1)			; -> st(0)=y*(0.2734510407e-4+y*(-0.2073370639e-5+y*0.2093887211e-6)); st(1)=y; st(2)=xx; st(3)=z; st(4)=ax; st(5)=x
	fld polynomial[128]		; -> st(0)=-0.1098628627e-2; st(1)=y*(0.2734510407e-4+y*(-0.2073370639e-5+y*0.2093887211e-6)); st(2)=y; st(3)=xx; st(4)=z; st(5)=ax; st(6)=x
	fwait
	faddp st(1),st			; -> st(0)=-0.1098628627e-2+y*(0.2734510407e-4+y*(-0.2073370639e-5+y*0.2093887211e-6)); st(1)=y; st(2)=xx; st(3)=z; st(4)=ax; st(5)=x
	fmul st,st(1)			; -> st(0)=y*(-0.1098628627e-2+y*(0.2734510407e-4+y*(-0.2073370639e-5+y*0.2093887211e-6))); st(1)=y; st(2)=xx; st(3)=z; st(4)=ax; st(5)=x
	fld1					; -> st(0)=1.0; st(1)=y*(-0.1098628627e-2+y*(0.2734510407e-4+y*(-0.2073370639e-5+y*0.2093887211e-6))); st(2)=y; st(3)=xx; st(4)=z; st(5)=ax; st(6)=x
	fwait
	faddp st(1),st			; -> st(0)=1.0+y*(-0.1098628627e-2+y*(0.2734510407e-4+y*(-0.2073370639e-5+y*0.2093887211e-6))); st(1)=y; st(2)=xx; st(3)=z; st(4)=ax; st(5)=x
	;----------------------------------------------------------------------------------
	; END ANS1 CALCULATION
	;----------------------------------------------------------------------------------
	
	; FPU state
	; -> st(0)=ans1; st(1)=y; st(2)=xx; st(3)=z; st(4)=ax; st(5)=x
	fxch st(1)				; -> st(0)=y; st(1)=ans1; st(2)=xx; st(3)=z; st(4)=ax; st(5)=x
	
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;THIS IS THE OTHER CONFLICT LINE. When I remove this line function result is not ok. It return 0
	; When one or both of conflict lines are removed function doesn't return the right value.
	invoke MessageBox,0, sstr$(3000000), 0 ,MB_OK
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;----------------------------------------------------------------------------------
	; ANS2 CALCULATION
	;----------------------------------------------------------------------------------
	fld polynomial[136]		; -> st(0)=0.934935152e-7; st(1)=y; st(2)=ans1; st(3)=xx; st(4)=z; st(5)=ax; st(6)=x
	fwait
	fmul st,st(1)			; -> st(0)=y*0.934935152e-7; st(1)=y; st(2)=ans1; st(3)=xx; st(4)=z; st(5)=ax; st(6)=x
	fld polynomial[144]		; -> st(0)=0.7621095161e-6; st(1)=y*0.934935152e-7; st(2)=y; st(3)=ans1; st(4)=xx; st(5)=z; st(6)=ax; st(7)=x
	fwait
	fsubrp st(1),st			; -> st(0)=0.7621095161e-6-y*0.934935152e-7; st(1)=y; st(2)=ans1; st(3)=xx; st(4)=z; st(5)=ax; st(6)=x
	fmul st,st(1)			; -> st(0)=y*(0.7621095161e-6-y*0.934935152e-7); st(1)=y; st(2)=ans1; st(3)=xx; st(4)=z; st(5)=ax; st(6)=x
	fld polynomial[152]		; -> st(0)=-0.6911147651e-5; st(1)=y*(0.7621095161e-6-y*0.934935152e-7); st(2)=y; st(3)=ans1; st(4)=xx; st(5)=z; st(6)=ax; st(7)=x
	fwait
	faddp st(1),st			; -> st(0)=-0.6911147651e-5+y*(0.7621095161e-6-y*0.934935152e-7); st(1)=y; st(2)=ans1; st(3)=xx; st(4)=z; st(5)=ax; st(6)=x
	fmul st,st(1)			; -> st(0)=y*(-0.6911147651e-5+y*(0.7621095161e-6-y*0.934935152e-7)); st(1)=y; st(2)=ans1; st(3)=xx; st(4)=z; st(5)=ax; st(6)=x
	fld polynomial[160]		; -> st(0)=0.1430488765e-3; st(1)=y*(-0.6911147651e-5+y*(0.7621095161e-6-y*0.934935152e-7)); st(2)=y; st(3)=ans1; st(4)=xx; st(5)=z; st(6)=ax; st(7)=x
	fwait
	faddp st(1),st			; -> st(0)=0.1430488765e-3+y*(-0.6911147651e-5+y*(0.7621095161e-6-y*0.934935152e-7)); st(1)=y; st(2)=ans1; st(3)=xx; st(4)=z; st(5)=ax; st(6)=x
	fmul st,st(1)			; -> st(0)=y*(0.1430488765e-3+y*(-0.6911147651e-5+y*(0.7621095161e-6-y*0.934935152e-7))); st(1)=y; st(2)=ans1; st(3)=xx; st(4)=z; st(5)=ax; st(6)=x
	fld polynomial[168]		; -> st(0)=-0.1562499995e-1; st(1)=y*(0.1430488765e-3+y*(-0.6911147651e-5+y*(0.7621095161e-6-y*0.934935152e-7))); st(2)=y; st(3)=ans1; st(4)=xx; st(5)=z; st(6)=ax; st(7)=x
	fwait
	faddp st(1),st			; -> st(0)=-0.1562499995e-1+y*(0.1430488765e-3+y*(-0.6911147651e-5+y*(0.7621095161e-6-y*0.934935152e-7))); st(1)=y; st(2)=ans1; st(3)=xx; st(4)=z; st(5)=ax; st(6)=x
	;----------------------------------------------------------------------------------
	; END ANS2 CALCULATION
	;----------------------------------------------------------------------------------
	
	fxch st(1)				; -> st(0)=y; st(1)=ans2; st(2)=ans1; st(3)=xx; st(4)=z; st(5)=ax; st(6)=x
	fstp st(0)				; We pop st(0)
	
	fld st(2)				; -> st(0)=xx; st(1)=ans2; st(2)=ans1; st(3)=xx; st(4)=z; st(5)=ax; st(6)=x
	fwait
	fsin					; -> st(0)=sin(xx); st(1)=ans2; st(2)=ans1; st(3)=xx; st(4)=z; st(5)=ax; st(6)=x
	fmulp st(1),st			; -> st(0)=sin(xx)*ans2; st(1)=ans1; st(2)=xx; st(3)=z; st(4)=ax; st(5)=x
	fmulp st(3),st			; -> st(0)=ans1; st(1)=xx; st(2)=z*sin(xx)*ans2; st(3)=ax; st(4)=x
	
	fxch st(1)				; -> st(0)=xx; st(1)=ans1; st(2)=z*sin(xx)*ans2; st(3)=ax; st(4)=x
	fcos					; -> st(0)=cos(xx); st(1)=ans1; st(2)=z*sin(xx)*ans2; st(3)=ax; st(4)=x
	fmulp st(1),st			; -> st(0)=cos(xx)*ans1; st(1)=z*sin(xx)*ans2; st(2)=ax; st(3)=x
	fsubrp st(1), st		; -> st(0)=cos(xx)*ans1-z*sin(xx)*ans2; st(1)=ax; st(2)=x

	fld polynomial[176]			; -> st(0)=0.636619772; st(1)=cos(xx)*ans1-z*sin(xx)*ans2; st(2)=ax; st(3)=x
	fwait
	fdivrp st(2),st			; -> st(0)=cos(xx)*ans1-z*sin(xx)*ans2; st(1)=0.636619772/ax; st(2)=x
	fxch st(1)				; -> st(0)=0.636619772/ax; st(1)=cos(xx)*ans1-z*sin(xx)*ans2; st(2)=x
	
	fsqrt					; -> st(0)=sqrt(0.636619772/ax); st(1)=cos(xx)*ans1-z*sin(xx)*ans2; st(2)=x
	fmulp st(1),st			; -> st(0)=sqrt(0.636619772/ax)*(cos(xx)*ans1-z*sin(xx)*ans2); st(1)=x
	
	@@end:
ret

Bessj0 endp